// Copyright (c) 2008  Future Technology Devices International Ltd.
//
// Module Name
//
//     JTAGDLLFuncUnit.pas
//
// Abstract:
//
//    Delphi program, to test the FTCJTAG.DLL. The FTCJTAG.DLL is used to control the FT2232H dual hi-speed device
//    and the FT4232H quad hi-speed device to simulate the Joint Test Action Group(JTAG) synchronous protocol to
//    communicate with I2C devices connected to FT2232H dual hi-speed devices and FT4232H quad hi-speed devices.
//
// Revision History:
//
//   16/09/08    Kirke Ashton    Created.
//
unit JTAGDLLFuncUnit;

interface

uses
  Windows, Classes, SysUtils, DateUtils, Dialogs, Controls, FT2232HJTAGUnit;

type FtcHiSpeedDeviceData = record
  DeviceName: String; //TDeviceName;
  LocationID: Dword;
  Channel: TChannel;
  HiSpeedDeviceType: Dword;
  ftHandle: Dword;
end;

type TBitLengths = array[0..1, 0..999] of integer;

type FtcBitLengths = record
  BitLengths: TBitLengths;
  NumBitLengths: integer;
end;

procedure DisplayFT2232HJTAGDllError(ProgFunctionName, DllFunctionName: String; ftStatus: FTC_STATUS);
function  GetJTAGDeviceNames(var DeviceNames: TStringList): Boolean;
function  OpenJTAGDevice(DeviceName: String; ClockDivisor: Dword): Boolean;
procedure CloseJTAGDevice(DeviceName: String; DeviceItemIndex: Dword);
function  JTAGScanIn(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToRead: Dword; pReadDataBuffer: PReadDataByteBuffer; pdwNumBytesReturned: PDword): Boolean;
function  JTAGScanOut(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToWrite: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword): Boolean;
function  JTAGScanInOut(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; pReadDataBuffer: PReadDataByteBuffer; pdwNumBytesReturned: PDword): Boolean;
procedure JTAGClearDeviceCmdSequence;
function  HexByteToStr(Dval : integer) : string;
function  JTAGExecuteDeviceCommandSequence(DeviceHandleIndex: Dword; var DisplayLines: TStringList): Boolean;
function  JTAGGenerateClockPulses(NumClockPulses: Dword; var DisplayLines: TStringList): Boolean;

var
  FT2232HJTAG: TFT2232HJTAG;
  FT2232HJTAGDll: Boolean;
  ftHandle: Dword;
  HiSpeedDevices : array[0..49] of FtcHiSpeedDeviceData;
  NumOpenedDevices: Dword;
  //BitLengths: array[0..1, 0..999] of integer;
  //NumBitLengths: integer;
  DeviceBitLengths : Array [0 .. 49] of FtcBitLengths;

const ScanInCmd = 0;
const ScanInOutCmd = 1;

implementation

procedure DisplayFT2232HJTAGDllError(ProgFunctionName, DllFunctionName: String; ftStatus: FTC_STATUS);
var
  ErrorMsg: TErrorMessage;
begin
  FT2232HJTAG.GetErrorCodeString(ENGLISH, ftStatus, @ErrorMsg, MAX_NUM_ERROR_MESSAGE_CHARS);

  ShowMessage('Program function name = ' + ProgFunctionName +
              ', FT2232HJTAG Dll function name = ' + DllFunctionName +
              ', Error message = ' + ErrorMsg);
end;

function GetJTAGDeviceNames(var DeviceNames: TStringList): Boolean;
var
  ftStatus: FTC_STATUS;
  NumHiSpeedDevices, DeviceIndex: Dword;
  DeviceName: TDeviceName;
  Channel: TChannel;
  LocationID: Dword;
  HiSpeedDeviceType: Dword;
begin
  if (FT2232HJTAGDll = True) then
  begin
    NumHiSpeedDevices := 0;
    ftStatus := FT2232HJTAG.GetNumHiSpeedDevices(@NumHiSpeedDevices);

    if (ftStatus = FTC_SUCCESS) then
    begin
      if (NumHiSpeedDevices > 0) then
      begin
        DeviceIndex := 0;
        repeat
          ftStatus := FT2232HJTAG.GetHiSpeedDeviceNameLocIDChannel(DeviceIndex, @DeviceName, MAX_NUM_DEVICE_NAME_CHARS, @LocationID, @Channel, MAX_NUM_CHANNEL_CHARS, @HiSpeedDeviceType);

          DeviceNames.Add(DeviceName);

          HiSpeedDevices[DeviceIndex].DeviceName := DeviceName;
          HiSpeedDevices[DeviceIndex].LocationID := LocationID;
          HiSpeedDevices[DeviceIndex].Channel := Channel;
          HiSpeedDevices[DeviceIndex].HiSpeedDeviceType := HiSpeedDeviceType;

          Inc(DeviceIndex);
        until ((DeviceIndex >= NumHiSpeedDevices) or (ftStatus <> FTC_SUCCESS));

        if (ftStatus <> FTC_SUCCESS) then
          DisplayFT2232HJTAGDllError('GetJTAGDeviceNames', 'GetHiSpeedDeviceNameLocIDChannel', ftStatus);
      end
      else
        ShowMessage('No FT2232H/FT4232HC hi-speed devices connected.');
    end
    else
      DisplayFT2232HJTAGDllError('GetJTAGDeviceNames', 'GetNumHiSpeedDevices', ftStatus);
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;
end;

function OpenJTAGDevice(DeviceName: String; ClockDivisor: Dword): Boolean;
var
  ftStatus: FTC_STATUS;
  LocationID: Dword;
  Channel: String;
  DeviceHandleIndex: Dword;
  ClockFrequencyHz: Dword;
  bDeviceStored: Boolean;
begin
  if (FT2232HJTAGDll = True) then
  begin
    for DeviceHandleIndex := 0 to 49 do
    begin
      if (HiSpeedDevices[DeviceHandleIndex].DeviceName = DeviceName) then
      begin
        LocationID := HiSpeedDevices[DeviceHandleIndex].LocationID;
        Channel := HiSpeedDevices[DeviceHandleIndex].Channel;
      end;
    end;

    ftStatus := FT2232HJTAG.OpenHiSpeedDevice(DeviceName, LocationID, Channel, @ftHandle);

    if (ftStatus = FTC_SUCCESS) then
    begin
      HiSpeedDevices[DeviceHandleIndex].ftHandle := ftHandle;

      ftStatus := FT2232HJTAG.InitDevice(ftHandle, ClockDivisor);

      if (ftStatus = FTC_SUCCESS) then
      begin
        ftStatus := FT2232HJTAG.GetClock(ClockDivisor, @ClockFrequencyHz);

        if (ftStatus = FTC_SUCCESS) then
        begin
          bDeviceStored := False;

          for DeviceHandleIndex := 0 to 49 do
          begin
            if ((HiSpeedDevices[DeviceHandleIndex].DeviceName = '') and
                (HiSpeedDevices[DeviceHandleIndex].ftHandle = 0) and (not bDeviceStored)) then
            begin
              HiSpeedDevices[DeviceHandleIndex].ftHandle := ftHandle;
              HiSpeedDevices[DeviceHandleIndex].DeviceName := DeviceName;
              bDeviceStored := True;
            end;
          end;

          Inc(NumOpenedDevices);
        end
        else
          DisplayFT2232HJTAGDllError('OpenHiSpeedDevice', 'GetClock', ftStatus);
      end
      else
        DisplayFT2232HJTAGDllError('OpenHiSpeedDevice', 'InitDevice', ftStatus);

      if (ftStatus <> FTC_SUCCESS) then
      begin
        FT2232HJTAG.Close(ftHandle);

        ftHandle := 0;
      end;
    end
    else
      DisplayFT2232HJTAGDllError('OpenHiSpeedDevice', 'OpenEx', ftStatus);
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;
end;

procedure CloseJTAGDevice(DeviceName: String; DeviceItemIndex: Dword);
var
  DeviceHandleIndex: Dword;
begin
  if (FT2232HJTAGDll = True) then
  begin
    if (fthandle <> 0) then
    begin
      FT2232HJTAG.Close(fthandle);

      if (NumOpenedDevices > 1) then
      begin
        if (DeviceItemIndex <> -1) then
        begin
          fthandle := 0;

          for DeviceHandleIndex := 0 to 49 do
          begin
            if (HiSpeedDevices[DeviceHandleIndex].DeviceName = DeviceName) then
            begin
              HiSpeedDevices[DeviceHandleIndex].ftHandle := ftHandle;
              HiSpeedDevices[DeviceHandleIndex].DeviceName := '';
            end;
          end;
        end;
      end
      else
      begin
        fthandle := 0;

        if (DeviceItemIndex <> -1) then
        begin
          for DeviceHandleIndex := 0 to 49 do
          begin
            if (HiSpeedDevices[DeviceHandleIndex].DeviceName = DeviceName) then
            begin
              HiSpeedDevices[DeviceHandleIndex].ftHandle := ftHandle;
              HiSpeedDevices[DeviceHandleIndex].DeviceName := '';
            end;
          end;
        end;
      end;

      if (NumOpenedDevices > 0) then
        Dec(NumOpenedDevices);
    end;
  end;
end;

function JTAGScanIn(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToRead: Dword; pReadDataBuffer: PReadDataByteBuffer; pdwNumBytesReturned: PDword): Boolean;
var
  ftStatus: FTC_STATUS;
  DeviceHandleIndex, DeviceIndex: Dword;
begin
  if (FT2232HJTAGDll = True) then
  begin
    if bBufferScanCommands then
    begin
      ftStatus := FT2232HJTAG.AddDeviceReadCmd(fthandle, bInstructionTestData, NumBitsToRead, RUN_TEST_IDLE_STATE);

      if (ftStatus = FTC_SUCCESS) then
      begin
        DeviceIndex := 0;
        for DeviceHandleIndex := 0 to 49 do
        begin
          if (HiSpeedDevices[DeviceHandleIndex].fthandle = fthandle) then
            DeviceIndex := DeviceHandleIndex;
        end;

        DeviceBitLengths[DeviceIndex].BitLengths[0][DeviceBitLengths[DeviceIndex].NumBitLengths] := NumBitsToRead;
        DeviceBitLengths[DeviceIndex].BitLengths[1][DeviceBitLengths[DeviceIndex].NumBitLengths] := ScanInCmd;
        Inc(DeviceBitLengths[DeviceIndex].NumBitLengths);
      end
      else
        DisplayFT2232HJTAGDllError('JTAGScanIn', 'AddDeviceReadCmd', ftStatus);
    end
    else
    begin
      ftStatus := FT2232HJTAG.Read(ftHandle, bInstructionTestData, NumBitsToRead,
                                   pReadDataBuffer, pdwNumBytesReturned, RUN_TEST_IDLE_STATE);

      if (ftStatus <> FTC_SUCCESS) then
        DisplayFT2232HJTAGDllError('JTAGScanIn', 'Read', ftStatus);
    end;
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;  
end;

function JTAGScanOut(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToWrite: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword): Boolean;
var
  ftStatus: FTC_STATUS;
  //DataByteBuffer: array[0..1] of Byte;
  //DllVersion: TDllVersion;
begin
  if (FT2232HJTAGDll = True) then
  begin
    //FT2232HJTAG.GetDllVersion(@DllVersion, MAX_NUM_DLL_VERSION_CHARS);
    //ShowMessage(DllVersion);

    if bBufferScanCommands then
    begin
      ftStatus := FT2232HJTAG.AddDeviceWriteCmd(fthandle, bInstructionTestData, NumBitsToWrite, pWriteDataBuffer, NumBytesToWrite, RUN_TEST_IDLE_STATE);
      //ftStatus := FT2232HJTAG.AddDeviceWriteCmd(fthandle, bInstructionTestData, NumBitsToWrite, @DataByteBuffer, NumBytesToWrite, RUN_TEST_IDLE_STATE);


      if (ftStatus <> FTC_SUCCESS) then
        DisplayFT2232HJTAGDllError('JTAGScanOut', 'AddDeviceWriteCmd', ftStatus);
    end
    else
    begin
      ftStatus := FT2232HJTAG.Write(ftHandle, bInstructionTestData, NumBitsToWrite,
                                    pWriteDataBuffer, NumBytesToWrite, RUN_TEST_IDLE_STATE);

     if (ftStatus <> FTC_SUCCESS) then
       DisplayFT2232HJTAGDllError('JTAGScanOut', 'Write', ftStatus);
     end;
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;    
end;

function JTAGScanInOut(bBufferScanCommands: Boolean; bInstructionTestData: LongBool; NumBitsToWriteRead: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumBytesToWrite: Dword; pReadDataBuffer: PReadDataByteBuffer; pdwNumBytesReturned: PDword): Boolean;
var
  ftStatus: FTC_STATUS;
  DeviceHandleIndex, DeviceIndex: Dword;
begin
  if (FT2232HJTAGDll = True) then
  begin
    if bBufferScanCommands then
    begin
      ftStatus := FT2232HJTAG.AddDeviceWriteReadCmd(fthandle, bInstructionTestData, NumBitsToWriteRead, pWriteDataBuffer, NumBytesToWrite, RUN_TEST_IDLE_STATE);

      if (ftStatus = FTC_SUCCESS) then
      begin
        DeviceIndex := 0;
        for DeviceHandleIndex := 0 to 49 do
        begin
          if (HiSpeedDevices[DeviceHandleIndex].fthandle = fthandle) then
            DeviceIndex := DeviceHandleIndex;
        end;

        DeviceBitLengths[DeviceIndex].BitLengths[0][DeviceBitLengths[DeviceIndex].NumBitLengths] := NumBitsToWriteRead;
        DeviceBitLengths[DeviceIndex].BitLengths[1][DeviceBitLengths[DeviceIndex].NumBitLengths] := ScanInOutCmd;

        Inc(DeviceBitLengths[DeviceIndex].NumBitLengths);
      end
      else
        DisplayFT2232HJTAGDllError('JTAGScanInOut', 'AddDeviceWriteReadCmd', ftStatus);
    end
    else
    begin
      ftStatus := FT2232HJTAG.WriteRead(ftHandle, bInstructionTestData,
                                        NumBitsToWriteRead, pWriteDataBuffer,
                                        NumBytesToWrite, pReadDataBuffer,
                                        pdwNumBytesReturned, RUN_TEST_IDLE_STATE);

      if (ftStatus <> FTC_SUCCESS) then
        DisplayFT2232HJTAGDllError('JTAGScanInOut', 'WriteRead', ftStatus);
    end;
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;
end;

procedure JTAGClearDeviceCmdSequence;
var
  ftStatus: FTC_STATUS;
  DeviceHandleIndex, DeviceIndex: Dword;
begin
  if (FT2232HJTAGDll = True) then
  begin
    DeviceIndex := 0;
    for DeviceHandleIndex := 0 to 49 do
    begin
      if (HiSpeedDevices[DeviceHandleIndex].fthandle = fthandle) then
        DeviceIndex := DeviceHandleIndex;
    end;
    
    DeviceBitLengths[DeviceIndex].NumBitLengths := 0;

    ftStatus := FT2232HJTAG.ClearDeviceCmdSequence(fthandle);

    if (ftStatus <> FTC_SUCCESS) then
      DisplayFT2232HJTAGDllError('JTAGClearDeviceCmdSequence', 'ClearDeviceCmdSequence', ftStatus);
  end;
end;

function HexByteToStr(Dval : integer) : string;
var i : integer;
retstr : string;
begin
retstr := '';
i := (Dval AND $F0) DIV $10;
case i of
  0 : retstr := retstr + '0';
  1 : retstr := retstr + '1';
  2 : retstr := retstr + '2';
  3 : retstr := retstr + '3';
  4 : retstr := retstr + '4';
  5 : retstr := retstr + '5';
  6 : retstr := retstr + '6';
  7 : retstr := retstr + '7';
  8 : retstr := retstr + '8';
  9 : retstr := retstr + '9';
  10 : retstr := retstr + 'A';
  11 : retstr := retstr + 'B';
  12 : retstr := retstr + 'C';
  13 : retstr := retstr + 'D';
  14 : retstr := retstr + 'E';
  15 : retstr := retstr + 'F';
end;
i := Dval AND $F;
case i of
  0 : retstr := retstr + '0';
  1 : retstr := retstr + '1';
  2 : retstr := retstr + '2';
  3 : retstr := retstr + '3';
  4 : retstr := retstr + '4';
  5 : retstr := retstr + '5';
  6 : retstr := retstr + '6';
  7 : retstr := retstr + '7';
  8 : retstr := retstr + '8';
  9 : retstr := retstr + '9';
  10 : retstr := retstr + 'A';
  11 : retstr := retstr + 'B';
  12 : retstr := retstr + 'C';
  13 : retstr := retstr + 'D';
  14 : retstr := retstr + 'E';
  15 : retstr := retstr + 'F';
end;
HexByteToStr := retstr;
end;

function JTAGExecuteDeviceCommandSequence(DeviceHandleIndex: Dword; var DisplayLines: TStringList): Boolean;
var
  ftStatus: FTC_STATUS;
  ExecftHandle: Dword;
  ReadDataBuffer: ReadDataByteBuffer;
  NumBytesReturned, NumDataBytesUsed: Dword;
  BitLengthsIndex, BitLength, NumBytes, ByteIndex: integer;
  ScanCmd: integer;
  OpenedDeviceName, DataMsg: String;
begin
  if (FT2232HJTAGDll = True) then
  begin
    ExecftHandle := HiSpeedDevices[DeviceHandleIndex].ftHandle;

    OpenedDeviceName := HiSpeedDevices[DeviceHandleIndex].DeviceName;

    ftStatus := FT2232HJTAG.ExecuteCmdSequence(ExecftHandle, @ReadDataBuffer, @NumBytesReturned);

    if (ftStatus = FTC_SUCCESS) then
    begin
      if (NumBytesReturned > 0) then
      begin
        NumDataBytesUsed := 0;

        for BitLengthsIndex := 0 to (DeviceBitLengths[DeviceHandleIndex].NumBitLengths - 1) do
        begin
          BitLength := DeviceBitLengths[DeviceHandleIndex].BitLengths[0][BitLengthsIndex];

          NumBytes := (BitLength div 8);

          if ((BitLength mod 8) > 0) then
            Inc(NumBytes);

          ScanCmd := DeviceBitLengths[DeviceHandleIndex].BitLengths[1][BitLengthsIndex];
          if (ScanCmd = ScanInCmd) then
            DataMsg := 'Scan In Cmd '
          else
            DataMsg := 'Scan In And Out Cmd ';

          DataMsg := OpenedDeviceName + ' - ' + DataMsg + 'Read back : ';

          for ByteIndex := (NumBytes - 1) downto 0 do
            DataMsg := DataMsg + HexByteToStr(ReadDataBuffer[(NumDataBytesUsed + ByteIndex)])  +' ';

          DisplayLines.Add(DataMsg);

          NumDataBytesUsed := (NumDataBytesUsed + NumBytes);
        end;
      end;
    end
    else
    begin
      if (ftStatus = FTC_NO_COMMAND_SEQUENCE) then
        DisplayLines.Add(OpenedDeviceName + ' - ' + 'No command sequence found.')
      else
        DisplayFT2232HJTAGDllError('JTAGExecuteDeviceCommandSequence', 'ExecuteCmdSequence', ftStatus);
    end;

    DeviceBitLengths[DeviceHandleIndex].NumBitLengths := 0;
  end;

  if (ftStatus = FTC_SUCCESS) then
    Result := True
  else
    Result := False;
end;

function JTAGGenerateClockPulses(NumClockPulses: Dword; var DisplayLines: TStringList): Boolean;
var
  ftStatus: FTC_STATUS;
begin
  if (FT2232HJTAGDll = True) then
  begin
    ftStatus := FT2232HJTAG.GenerateClockPulses(fthandle, NumClockPulses);

    if (ftStatus = FTC_SUCCESS) then
      DisplayLines.Add('Successfully generated clock pulses.')
    else
    begin
      DisplayLines.Add('Failed to generate clock pulses.');

      DisplayFT2232HJTAGDllError('JTAGGenerateClockPulses', 'GenerateClockPulses', ftStatus);
    end;
  end;
end;

end.
